#!/usr/bin/perl
use warnings;
use strict;
use v5.20;
use utf8;

use MIME::Lite;
use Getopt::Long qw/GetOptions/;
use Encode qw/encode_utf8/;
use File::Slurp qw/read_file/;

use lib '.';
use Frag::Misc::SleepyDaemon 'run';

binmode STDOUT, ':utf8';
binmode STDERR, ':utf8';

my %options;
GetOptions( \%options,
    'loop|l',
    'dry-run|n',
) or die;

# CONFIGURATION
# Put mail recipients' logins (or addresses) into ‹workdir›/mailto, one per line
# Spams ‹subject›@fi.muni.cz unless the mailto file contains a recipient.

%options = ( %options,
        subject => 'pv264',
        dbhost  => 'frag-db.fi.muni.cz',
        workdir => '/home/xmatous3/pv264/mailer/submission-announcer',
        job     => \&check_and_mail,
    );


run( \%options );

exit 0;

##############################################################################

sub check_and_mail {
    my ($dbh, $stamp, $options) = @_;
    my $sth = $dbh->prepare( <<~'EOF' );
    with passed as
            (select eo.eval_id from frag.eval_out as eo
                where eo.group = 'verity'
                group by eval_id
                having every(eo.passed)),
         newest as
            (select distinct on (s.author, s.assignment_id)
                s.author, s.assignment_id, s.id as sid, s.stamp
                from passed join frag.eval as e on passed.eval_id = e.id
                join frag.submission as s on submission_id = s.id
                order by s.author, s.assignment_id, s.stamp desc),
         teachers as
            (select teacher from frag.teacher_list),
         claims as
            (select login as assignee, author, assignment_id as aid
                from frag.review
                join frag.submission as s on s.id = submission_id
                join frag.person on person.id = reviewer
                where reviewer in (table teachers)),
         recent as
            (select *
                from newest
                where newest.author not in (table teachers)
                    and newest.stamp > to_timestamp( ? )
                ),
         subjoin as
            (select recent.*, greatest( recent.assignment_id, c.aid ) as aid, greatest( recent.author, c.author ) as uid, assignee
                from recent
                full outer join claims as c
                    on (c.author, c.aid) = (recent.author, recent.assignment_id))
    select convert_from(p.login, 'UTF8') as login, p.name,
        j.sid as sub_id, a.name as hw, j.stamp as submitted, assignee
        from subjoin as j
        join frag.assignment as a on a.id = j.aid
        join frag.person as p on p.id = j.uid
        order by j.stamp asc, assignee, a.name
    EOF

    $sth->execute( $stamp )
        or die;

    my @new_subs;
    my @old_maps;

    while ( my $r = $sth->fetchrow_hashref ) {
        if ( defined $r->{sub_id} ) {
            my $assignee = $r->{assignee} ? "for $r->{assignee}": 'AVAILABLE';
            my $s = "    • $r->{sub_id} by $r->{name} ($r->{login}) – $r->{hw} on $r->{submitted} ($assignee)";
            push @new_subs, $s;
        } else {
            my $map = "    • $r->{assignee} reviews $r->{login}'s $r->{hw}";
            push @old_maps, $map;
        }
    }

    unless ( @new_subs ) {
        say "No new submissions since the last check." if -t STDOUT;
        return;
    }

    $" = ",\n";

    my $body = <<~EOF;
    Hello,

    There are new passing PV264 submissions:

    @new_subs.

    Handy commands to start reviewing:

        \$ ‹frag› review --checkout --submission-id ‹ID›
        \$ ‹frag› review --checkout --redo                 # resubmits for you

    By the way, there are currently these additional mappings:

    @old_maps.

    Regards
    -- 
    PV264 Mailer
    EOF

    say $body if -t STDOUT;

    my @to;
    foreach ( read_file( "$options->{workdir}/mailto", err_mode => 'silent' ) ) {
        s/^\s*//;
        s/#.*//;
        s/\s*$//;
        next unless $_;
        s/$/\@fi.muni.cz/ unless /\@/;
        push @to, $_;
    }
    push @to, "$options->{subject}\@fi.muni.cz" unless @to;


    for my $to ( @to ) {
        my $subj = '[' . uc( $options->{subject} ) . '] ' . @new_subs
            . ' new submission' . ((@new_subs > 1) ? 's' : '') . ' to review';
        my $msg = MIME::Lite->new(
            From    => $options->{from},
            To      => $to,
            Subject => $subj,
            Data    => encode_utf8( $body )
        );
        $msg->attr( 'content-type' => 'text/plain; charset=utf-8');
        $msg->send( 'sendmail', '/usr/lib/sendmail -t -X ""') unless $options->{'dry-run'};
        say "To: $to" if -t STDOUT;
    }
    say "(No changes made, this is a dry run)" if $options->{'dry-run'};
}
